package cloud

import "sort"

//------------------------------------//
// JSON File Storing       						//
//------------------------------------//

// JSONConnectionInfo is a type used for writing connection info to a json file.
type JSONConnectionInfo struct {
	GatewayID      int     `json:"gateway_id"`
	SignalStrength float64 `json:"signal_strength"`
	Timestamp      string  `json:"timestamp"`
}

// JSONMessage is a type used for writing messages to a json file.
type JSONMessage struct {
	MessageID      int                  `json:"message_id"`
	ConnectionInfo []JSONConnectionInfo `json:"connection_info"`
}

// JSONSensor is a type used for writing messages to a json file.
type JSONSensor struct {
	SensorID int           `json:"sensor_id"`
	Messages []JSONMessage `json:"messages"`
}

// JSONTop is a type used for holding all sensors for writing to a json file.
type JSONTop struct {
	Sensors []JSONSensor `json:"sensors"`
}

func (c *Cloud) prepareMessages() JSONTop {

	t := JSONTop{}

	// Unfortunately, Golang does not guarantee the order of keys in a map.
	// So we need to sort them manually before writing to .json file.
	sensors := make(map[int][]int)

	// Messages V2
	for key := range c.messages {
		sensors[key.SensorID] = append(sensors[key.SensorID], key.MessageID)
	}

	// Now we have all sensor IDs, together with the message IDs they used.
	// Sort them and then insert them into the JSON!

	for sensorID := range sensors {
		sort.Ints(sensors[sensorID])

		s := JSONSensor{SensorID: sensorID}

		for _, messageID := range sensors[sensorID] {
			key := MessageKey{sensorID, messageID}
			m := JSONMessage{MessageID: messageID}

			for _, connInfo := range c.messages[key].Gateways {

				conn := JSONConnectionInfo{connInfo.GatewayID, connInfo.SignalStrength, connInfo.Timestamp}

				m.ConnectionInfo = append(m.ConnectionInfo, conn)
			}
			s.Messages = append(s.Messages, m)
		}
		t.Sensors = append(t.Sensors, s)
	}

	/*
		b, _ := json.Marshal(t)
		err := ioutil.WriteFile(filename, b, 0644)
		if err != nil {
			panic(err)
		}
	*/

	return t

}
