package cloud

import (
	"github.com/orjansv/master-code/simulator/sensor"
)

func (c *Cloud) selectGatewaySignalStrength(msg sensor.Message, newKey ConnectionKey) {

	currentBestCandidate := ConnectionKey{msg.SensorID, c.responsibleGateway[msg.SensorID].NextResponsible}

	// To avoid changing when they are very close, we only change if it is more than a minimum higher.
	if c.connections[newKey].SignalStrength.Average() > c.connections[currentBestCandidate].SignalStrength.Average()+c.config.MinSignalStrengthDifference {
		c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
	} else if newKey.GatewayID == c.responsibleGateway[msg.SensorID].ID {

		// A retry might make the next optimal candidate less ideal than we initially thought, so re-check the improvement for the current optimal.
		// If the improvement is not as good as hoped, keep the current responsible gateway.
		if c.connections[newKey].SignalStrength.Average() >= c.connections[currentBestCandidate].SignalStrength.Average()-c.config.MinSignalStrengthDifference {
			c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
		}
	}
}

func (c *Cloud) selectGatewayLowestRetryCount(msg sensor.Message, newKey ConnectionKey) {

	currentBestCandidate := ConnectionKey{msg.SensorID, c.responsibleGateway[msg.SensorID].NextResponsible}

	if c.retries[newKey].TotalCount() < c.retries[currentBestCandidate].TotalCount() {
		c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
	} else if c.retries[newKey].TotalCount() == c.retries[currentBestCandidate].TotalCount() {
		// Keep current gateway if two candidates are even.
		if newKey.GatewayID == c.responsibleGateway[msg.SensorID].ID {
			c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
		} else if currentBestCandidate.GatewayID != c.responsibleGateway[msg.SensorID].ID {
			// Use signal strength in case of a draw.
			if c.connections[newKey].SignalStrength.Average() > c.connections[currentBestCandidate].SignalStrength.Average() {
				c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
			}
		}
	}
}

func (c *Cloud) selectGatewayReliability(msg sensor.Message, newKey ConnectionKey) {
	currentBestCandidate := ConnectionKey{msg.SensorID, c.responsibleGateway[msg.SensorID].NextResponsible}

	weightedRelNew := c.calculateWeightedReliability(newKey, c.sensor.BoostModeEnabled)
	weightedRelNext := c.calculateWeightedReliability(currentBestCandidate, c.sensor.BoostModeEnabled)

	if weightedRelNew > weightedRelNext+c.config.MinReliabilityImprovement {
		c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
	} else if newKey.GatewayID == c.responsibleGateway[msg.SensorID].ID {
		// Keep current responsible gateway if the improvement was not good enough.
		if weightedRelNew >= weightedRelNext-c.config.MinReliabilityImprovement {
			c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
		}
	}
}

func (c *Cloud) selectGatewayEnergy(msg sensor.Message, newKey ConnectionKey) {

	currentBestCandidate := ConnectionKey{msg.SensorID, c.responsibleGateway[msg.SensorID].NextResponsible}

	energyNew := c.connections[newKey].ExpectedEnergyConsumption.Average()
	energyOld := c.connections[currentBestCandidate].ExpectedEnergyConsumption.Average()

	if energyNew*c.config.MinEnergyImprovement < energyOld {
		c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
	} else if newKey.GatewayID == c.responsibleGateway[msg.SensorID].ID {
		// A retry might make the next optimal less ideal than we initially thought, so re-check the improvement for the current optimal.
		if energyNew <= energyOld*c.config.MinEnergyImprovement {
			c.responsibleGateway[msg.SensorID].NextResponsible = newKey.GatewayID
		}
	}
}
