package cloud

import (
	"time"

	"github.com/orjansv/master-code/simulator/common"
	"github.com/orjansv/master-code/simulator/sensor"
	"github.com/orjansv/master-code/utils"
)

// Config has all settings that can be set in the json config file.
type Config struct {
	MinEnergyImprovement        float64
	MinSignalStrengthDifference float64
	MinReliabilityImprovement   float64
	WeightUplink                float64
	WeightDownlink              float64
}

// MessageKey is used for uniquely identifying messages.
type MessageKey struct {
	SensorID  int
	MessageID int
}

// Event is the event that a message arrived.
type Event struct {
	SensorID   int
	Ciphertext map[string]bool
	Gateways   []ConnectionInfo
}

// ConnectionInfo is a type used to keep track of the state of connections.
type ConnectionInfo struct {
	GatewayID      int
	SignalStrength float64
	Timestamp      string
}

// ConnectionStatus keeps track of the status of a connection over time.
type ConnectionStatus struct {
	SignalStrength            *utils.MovingAverage
	ExpectedEnergyConsumption *utils.MovingAverage
}

// ResponsibleGateway keeps track of useful information about the responsible gateway for a given sensor.
type ResponsibleGateway struct {
	ID               int
	RepliesAvailable int
	Replied          bool
	RepliedBoost     bool
	NextResponsible  int
}

// ConnectionKey is used for identifying connections.
type ConnectionKey struct {
	SensorID, GatewayID int
}

// SensorInfo contains the information that the cloud should know about the sensors.
type SensorInfo struct {
	TransmissionPowers []float64
	MaxAttempts        int
	DataRate           float64
	BoostModeEnabled   bool
	WaitingTime        time.Duration
}

// Channels is a struct that holds all channels for the cloud.
type Channels struct {
	InChannel    chan sensor.Message
	OutChans     []chan common.CommandMessage
	StopChan     chan string
	MessagesChan chan JSONTop
}

// UpLink is used for monitoring the stability of a connection from one sensor to all its gateways.
type UpLink struct {
	MessagesReceivedTotal      map[int]*utils.MovingWindowFloat
	MessagesReceivedConnection map[ConnectionKey]*utils.MovingWindowFloat
	Available                  map[ConnectionKey]bool
}

// DownLink is used for monitoring the reliability of the connections from gateways to sensors.
type DownLink struct {
	retriesCaused, repliesSent *utils.MovingWindowFloat
}
