package cloud

// These are the available gateway selection methods for the cloud.
const (
	SelectionMethodStrongestSignal int = iota
	SelectionMethodHighestReliability
	SelectionMethodLowestEnergyConsumption
	SelectionMethodLowestRetryCount
)

// SelectionMethodStringToType is used for converting method string to value.
var SelectionMethodStringToType = map[string]int{
	"strongest_signal":          SelectionMethodStrongestSignal,
	"highest_reliability":       SelectionMethodHighestReliability,
	"lowest_energy_consumption": SelectionMethodLowestEnergyConsumption,
	"lowest_retry_count":        SelectionMethodLowestRetryCount,
}

// SelectionMethodsString are the available methods for selecting responsible gateways as readable strings.
var SelectionMethodsString = map[int]string{
	SelectionMethodStrongestSignal:         "strongest_signal",
	SelectionMethodHighestReliability:      "highest_reliability",
	SelectionMethodLowestEnergyConsumption: "lowest_energy_consumption",
	SelectionMethodLowestRetryCount:        "lowest_retry_count",
}

var selectionReasons = map[int]string{
	SelectionMethodLowestEnergyConsumption: "lower energy consumption.",
	SelectionMethodStrongestSignal:         "higher average signal strength.",
	SelectionMethodLowestRetryCount:        "lower retry count.",
	SelectionMethodHighestReliability:      "higher connection reliability.",
}
