package simulator

import "github.com/orjansv/master-code/utils"
import "github.com/orjansv/master-code/simulator/simtime"

// These are used to unmarshal the json config file.

// ConfigJSON is the top level struct for the configuration file.
type ConfigJSON struct {
	Sensors     SensorJSON     `json:"sensors"`
	Gateways    GatewayJSON    `json:"gateways"`
	Cloud       CloudJSON      `json:"cloud"`
	Connections ConnectionJSON `json:"connections"`
	Simulation  SimulationJSON `json:"simulation"`
}

// SensorJSON holds all sensor related settings from the configuration file.
type SensorJSON struct {
	Coordinates       []utils.Coordinates   `json:"coordinates"`
	MultiCoordinates  [][]utils.Coordinates `json:"multi_coordinates"`
	TransmissionPower []float64             `json:"transmission_power"`
	TPC               bool                  `json:"tpc"`
	SendTPC           bool                  `json:"send_tpc"`
	Sensitivity       float64               `json:"sensitivity"`
	MaxAttempts       int                   `json:"max_attempts"`
	PayloadSize       int                   `json:"payload_size"`
	AddressSize       int                   `json:"address_size"`
	CiphertextSize    int                   `json:"ciphertext_size"`
	StartInterval     int64                 `json:"start_interval"`
	SendInterval      int                   `json:"send_interval"`
	DataRate          int                   `json:"data_rate"`
	WaitingTime       int                   `json:"waiting_time"`
}

// ConnectionJSON holds all connection related settings from the configuration file.
type ConnectionJSON struct {
	Distributions       ConnJSON `json:"distributions"`
	BoostDistributions  ConnJSON `json:"boost_distributions"`
	BlockProbability    int      `json:"block_prob"`
	DynamicLoss         int      `json:"dynamic_loss"`
	SymmetricConnection bool     `json:"symmetric_connection"`
	NormalRange         float64  `json:"normal_range"`
	AdjustFSPL          bool     `json:"adjust_fspl"`
	BoostRange          float64  `json:"boost_range"`
	BoostCost           float64  `json:"boost_cost"`
}

// SimulationJSON holds all general settings from the configuration file.
type SimulationJSON struct {
	SimulationSpeed int64                 `json:"simulation_speed"`
	Frequency       float64               `json:"frequency"`
	Seed            string                `json:"seed"`
	Duration        simtime.SimulatedTime `json:"duration"`
	WindowSize      int                   `json:"window_size"`
	Title           string                `json:"title"`
	Boost           bool                  `json:"boost"`
	BitCost         float64               `json:"bit_cost"`
}

// GatewayJSON holds all gateway related settings from the configuration file.
type GatewayJSON struct {
	Coordinates       []utils.Coordinates   `json:"coordinates"`
	MultiCoordinates  [][]utils.Coordinates `json:"multi_coordinates"`
	TransmissionPower float64               `json:"transmission_power"`
	Sensitivity       float64               `json:"sensitivity"`
}

// CloudJSON holds all cloud related settings from the configuration file.
type CloudJSON struct {
	MinEnergyImprovement        float64 `json:"minimum_energy_improvement"`
	MinSignalStrengthDifference float64 `json:"minimum_signal_strength_difference"`
	MinReliabilityImprovement   float64 `json:"minimum_reliability_improvement"`
	SelectionMethod             string  `json:"selection_method"`
	WeightDownlink              float64 `json:"weight_downlink"`
	WeightUplink                float64 `json:"weight_uplink"`
}

// DurationJSON is a specific struct for holding simulation duration information.
type DurationJSON struct {
	Days    int `json:"days"`
	Hours   int `json:"hours"`
	Minutes int `json:"minutes"`
	Seconds int `json:"seconds"`
}

// ConnJSON is used for holding connection type distribution.
type ConnJSON struct {
	Standard float64 `json:"standard"`
	Binary   float64 `json:"binary"`
	Dynamic  float64 `json:"dynamic"`
}

// These are used for writing a new json file for plotting the results.

// GridJSON is used for creating a json file that is used for plotting the grid in python.
type GridJSON struct {
	Sensors     []utils.Coordinates `json:"sensors"`
	Gateways    []utils.Coordinates `json:"gateways"`
	Connections connectionsJSON     `json:"connections"`
	Number      int                 `json:"number"`
}

type connectionsJSON struct {
	Standard []LinesJSON `json:"standard"`
	Dynamic  []LinesJSON `json:"dynamic"`
	Binary   []LinesJSON `json:"binary"`
	Boost    []LinesJSON `json:"boost"`
}

// LinesJSON holds the coordinates for a line.
type LinesJSON struct {
	X [2]float64 `json:"x"`
	Y [2]float64 `json:"y"`
}
