package simtime

import (
	"fmt"
	"time"
)

// SimulatedTime is what the name says.
type SimulatedTime struct {
	Days    int64 `json:"days"`
	Hours   int64 `json:"hours"`
	Minutes int64 `json:"minutes"`
	Seconds int64 `json:"seconds"`
}

// SubInMinutes returns the subtraction of sometime from s in minutes
func (s SimulatedTime) SubInMinutes(someTime SimulatedTime) int64 {
	result := s.Days*24*60 - someTime.Days*24*60
	result += s.Hours*60 - someTime.Hours*60
	result += s.Minutes - someTime.Minutes

	return result
}

// SubInHours returns the subtraction of someTime in simulated time in hours.
func (s SimulatedTime) SubInHours(someTime SimulatedTime) int64 {
	result := s.Days*24 - someTime.Days*24
	result += s.Hours - someTime.Hours

	return result
}

// Subtract returns the subtraction of the input time from the current time.
func (s SimulatedTime) Subtract(someTime SimulatedTime) SimulatedTime {
	return SimulatedTime{
		Days:    s.Days - someTime.Days,
		Hours:   s.Hours - someTime.Hours,
		Minutes: s.Minutes - someTime.Minutes,
		Seconds: s.Seconds - someTime.Seconds,
	}
}

// ToString returns simulated time as a nicely formatted string.
func (s SimulatedTime) ToString() string {
	return fmt.Sprintf("Day: %d - %02d:%02d:%02d", s.Days, s.Hours, s.Minutes, s.Seconds)
}

var multiplier float64 // for converting passed time to seconds

// Minute is the time of one simulated minute in real time
var Minute time.Duration // one minute simulated time in actual time

// Second is the time of one simulated second in real time
var Second time.Duration

// SetSimulationSpeed sets the amount of seconds one real time second should be in the simulation
func SetSimulationSpeed(speed int64) {

	multiplier = float64(speed) / 1000

	Minute = time.Duration((60*1000)/speed) * time.Millisecond

	Second = Minute / 60.0

}

// GetSimulationTime converts real time to simulated time.
func GetSimulationTime(t time.Time) SimulatedTime {

	passed := (float64)(time.Now().Sub(t) / time.Millisecond)
	s := (int64)(passed * multiplier)
	m := s / 60
	h := m / 60
	d := h / 24
	h = h - d*24
	m = m - h*60 - d*24*60
	s = s - m*60 - h*3600 - d*86400

	return SimulatedTime{Days: d, Hours: h, Minutes: m, Seconds: s}
}
