package simulator

import (
	"time"

	"github.com/orjansv/master-code/simulator/cloud"
	"github.com/orjansv/master-code/simulator/common"
	"github.com/orjansv/master-code/simulator/sensor"
	"github.com/orjansv/master-code/simulator/simtime"
	"github.com/orjansv/master-code/simulator/watchdog"
	"github.com/orjansv/master-code/utils"
)

type simulatorConfig struct {
	numSensors, numGateways int
	normalRange             float64
	selectionMethod         int
	frequency               float64
	duration                simtime.SimulatedTime
	seed                    int64
	windowSize              int
	blockProbability        int
	symmetricConnections    bool
	sensors                 sensorConfig
	gateways                gatewayConfig
	cloud                   cloud.Config
	boostRange              float64 // split these to connconf
	boostCost               float64
	boostEnabled            bool
	fsplAdjustment          float64
	fsplAdjustmentBoost     float64
	dynamicLoss             int
	bitCost                 float64
}

type sensorConfig struct {
	sensitivity       float64
	transmissionPower []float64
	sensorType        int
	maxAttempts       int
	positions         []utils.Coordinates
	energyConsumption float64
	payloadSize       int
	addressSize       int
	ciphertextSize    int
	startInterval     time.Duration
	sendingInterval   time.Duration
	waitingTime       time.Duration
	dataRate          int
	sendTPC           bool
}

type gatewayConfig struct {
	sensitivity       float64
	transmissionPower float64
	positions         []utils.Coordinates
}

type simulatorChannels struct {
	cloud2GatewayChans  []chan common.CommandMessage
	gateway2CloudChan   chan sensor.Message
	gateway2SensorChans []chan sensor.Message
	sensor2GatewayChans []chan sensor.Message
	watchdogMessageChan chan watchdog.WatchdogMessage
	watchDogStopChan    chan string
	messagesChan        chan cloud.JSONTop
	energyStatsChan     chan watchdog.EnergyStats
	sensorStopChans     []chan int
	gatewayStopChans    []chan int
}

// SimulationResults is used to hold all results from one single simulation.
type SimulationResults struct {
	Messages    cloud.JSONTop
	Performance watchdog.EnergyStats
	Connections GridJSON
}
