import matplotlib.pyplot as plt
import math
import json
import os.path
import sys
import numpy as np
from matplotlib import rc

def label_bar(ax, bars, text_format, is_inside=True, **kwargs):
  """
  Attach a text label to each bar displaying its y value
  """
  max_y_value = max(bar.get_height() for bar in bars)

  y_lims = ax.get_ylim()

  if is_inside:
      #distance = max_y_value * 0.05
      distance = (y_lims[1] - y_lims[0])*0.05
  else:
      #distance = max_y_value * 0.01
      distance = (y_lims[1] - y_lims[0])*0.01

  for bar in bars:
    text = text_format.format(bar.get_height())
    text_x = bar.get_x() + bar.get_width() / 2
    if is_inside:
        text_y = bar.get_height() - distance
    else:
        text_y = bar.get_height() + distance

    ax.text(text_x, text_y, text, ha='center', va='bottom', **kwargs)


if len(sys.argv) > 3:
  print("Error: This program takes one or two arguments: name of json file and optionally -a for plotting all figures.")
  exit(1)

if sys.argv[1].endswith('json') != True:
  print("Error: The file has to be a .json file.")
  exit(1)

plotAll = False
if len(sys.argv) == 3:
  if sys.argv[2] == "-a":
    plotAll = True
  else:
    print("Unknown option " + sys.argv[2])

# Name without extension and "_stats".
tmp = os.path.splitext(sys.argv[1])[0]
name = tmp.split('_')[0]

json_data=open(sys.argv[1]).read()
data = json.loads(json_data)

dataTitle = data["Title"]

# LaTeX fonts :-)
rc('font', **{'family': 'serif', 'serif': ['Computer Modern']})
rc('text', usetex=True)

# For numbers above bars
value_format = "{}"  # displaying values as percentage with one fractional digit
value_format_float = "{:.4f}"

bar_width = 0.75
width=8.0
height=6.0

index = np.arange(len(data["Benchmarks"]))

names = []
meanRetries = []
stdRetries = []
meanEnergy = []
stdEnergy = []
meanBoosts = []
stdBoosts = []

totalBoosts = 0

# Extract values from JSON and calculate mean and std
for i, val in enumerate(data["Benchmarks"]):
  names = names + [val["Method"].encode('ascii', 'ignore')]
  e = np.array(val["Energy"])
  r = np.array(val["Retries"])
  b = np.array(val["Boosts"])
  totalBoosts += val["TotalBoost"]
  meanRetries.append(np.mean(r))
  stdRetries.append(np.std(r))
  meanEnergy.append(np.mean(e))
  stdEnergy.append(np.std(e))
  meanBoosts.append(np.mean(b))
  stdBoosts.append(np.std(b))

# Sort so we plot consistent for the thesis
sortedNames = sorted(names)
sortedMeanRetries = []
sortedStdRetries = []
sortedMeanEnergy = []
sortedStdEnergy = []
sortedMeanBoosts = []
sortedStdBoosts = []
sortedRsdRetries, sortedRsdEnergy, sortedRsdBoosts = [], [], []
for i, n in enumerate(sortedNames):
  indx = names.index(n)
  sortedMeanRetries.append(meanRetries[indx])
  sortedStdRetries.append(stdRetries[indx])
  sortedRsdRetries.append(stdRetries[indx]/meanRetries[indx])
  sortedMeanEnergy.append(meanEnergy[indx])
  sortedStdEnergy.append(stdEnergy[indx])
  sortedRsdEnergy.append(stdEnergy[indx]/meanEnergy[indx])
  sortedMeanBoosts.append(meanBoosts[indx])
  sortedStdBoosts.append(stdBoosts[indx])
  sortedRsdBoosts.append(stdBoosts[indx]/meanBoosts[indx])

print("Retries")
print("Energy")
print("Boosts")
print("{0} & {1} & {2} & {3}".format(round(sortedMeanRetries[0], 2), round(sortedMeanRetries[1], 2), round(sortedMeanRetries[2], 2), round(sortedMeanRetries[3], 2)))
print("{0} & {1} & {2} & {3}".format(round(sortedStdRetries[0], 2), round(sortedStdRetries[1], 2), round(sortedStdRetries[2], 2), round(sortedStdRetries[3], 2)))
print("{0} & {1} & {2} & {3}".format(round(sortedRsdRetries[0], 2), round(sortedRsdRetries[1], 2), round(sortedRsdRetries[2], 2), round(sortedRsdRetries[3], 2)))
print("{0} & {1} & {2} & {3}".format(round(sortedMeanEnergy[0], 2), round(sortedMeanEnergy[1], 2), round(sortedMeanEnergy[2], 2), round(sortedMeanEnergy[3], 2)))
print("{0} & {1} & {2} & {3}".format(round(sortedStdEnergy[0], 2), round(sortedStdEnergy[1], 2), round(sortedStdEnergy[2], 2), round(sortedStdEnergy[3], 2)))
print("{0} & {1} & {2} & {3}".format(round(sortedRsdEnergy[0], 2), round(sortedRsdEnergy[1], 2), round(sortedRsdEnergy[2], 2), round(sortedRsdEnergy[3], 2)))
print("{0} & {1} & {2} & {3}".format(round(sortedMeanBoosts[0], 2), round(sortedMeanBoosts[1], 2), round(sortedMeanBoosts[2], 2), round(sortedMeanBoosts[3], 2)))
print("{0} & {1} & {2} & {3}".format(round(sortedStdBoosts[0], 2), round(sortedStdBoosts[1], 2), round(sortedStdBoosts[2], 2), round(sortedStdBoosts[3], 2)))
print("{0} & {1} & {2} & {3}".format(round(sortedRsdBoosts[0], 2), round(sortedRsdBoosts[1], 2), round(sortedRsdBoosts[2], 2), round(sortedRsdBoosts[3], 2)))


energyUnit = 'mWs'
# Plot retries and energy consumption
if plotAll:
  fig, ax = plt.subplots()

  for i, n in enumerate(sortedNames):
    bar = ax.bar(i, sortedMeanRetries[i], bar_width, label=n)
    label_bar(ax, bar, value_format, is_inside=False, color="black")

  ax.set_xticks(index)
  ax.set_xticklabels((sortedNames))

  ax.set_ylabel('Retries')

  minR = min(sortedMeanRetries)
  maxR = max(sortedMeanRetries)
  diffR = maxR - minR

  ax.set_ylim([0, maxR + diffR])

  for label in ax.get_xmajorticklabels() + ax.get_xmajorticklabels():
      label.set_rotation(20)
      label.set_horizontalalignment("right")

  ax.legend(fancybox=False, edgecolor='black')
  plt.title(dataTitle.title() + " - Retries")
  plt.savefig(name + '_retries.pdf', bbox_inches='tight')


  # Plot energy consumption
  convertUnit = False
  for i, energy in enumerate(sortedMeanEnergy):
    if energy >= 1000:
      convertUnit = True
      break

  #if convertUnit == True:
  #  energyUnit = 'Ws'
  #  for i, energy in enumerate(sortedMeanEnergy):
  #    sortedMeanEnergy[i] = energy/1000

  fig, ax = plt.subplots()

  minE = min(sortedMeanEnergy)
  maxE = max(sortedMeanEnergy)
  diff = maxE - minE

  ax.set_ylim([minE - diff, maxE + diff])

  for i, n in enumerate(sortedNames):
    bar = ax.bar(i, sortedMeanEnergy[i], bar_width, label=n)
    label_bar(ax, bar, value_format_float, is_inside=False, color="black")

  ax.set_xticks(index)
  ax.set_xticklabels((sortedNames))

  ax.set_ylabel('Energy Consumption [' + energyUnit + ']')

  for label in ax.get_xmajorticklabels() + ax.get_xmajorticklabels():
      label.set_rotation(20)
      label.set_horizontalalignment("right")

  ax.legend(fancybox=False, edgecolor='black')
  plt.title(dataTitle.title() + " - Energy Consumption")
  plt.savefig(name + '_energy.pdf', bbox_inches='tight')

# Plot both in same figure

# LaTeX fonts :-)
rc('font', **{'family': 'serif', 'serif': ['Computer Modern'], 'size': 12})
rc('text', usetex=True)

if totalBoosts > 0:
  fig, (ax1, ax2, ax3) = plt.subplots(3, figsize=(6.53,4))
else:
  fig, (ax1, ax2) = plt.subplots(2, figsize=(6.53,3.5))

ax1.set_title(dataTitle.title())

minR = sortedMeanRetries[0]
maxR = 0
for i, s in enumerate(sortedStdRetries):
  maxR = max((s + sortedMeanRetries[i]), maxR)
  minR = min((sortedMeanRetries[i] - s), minR)

if minR < 0:
  minR = 0

ax1.set_ylim([minR*0.8, maxR*1.2])

minE = sortedMeanEnergy[0]
maxE = 0
for i, s in enumerate(sortedStdEnergy):
  maxE = max((s + sortedMeanEnergy[i]), maxE)
  minE = min((sortedMeanEnergy[i] - s), minE)

if minE < 0:
  minE = 0

ax2.set_ylim([minE*0.8, maxE*1.2])

minB = sortedMeanBoosts[0]
maxB = 0
for i, s in enumerate(sortedStdBoosts):
  maxB = max((s + sortedMeanBoosts[i]), maxB)
  minB = min((sortedMeanBoosts[i] - s), minB)

if minB < 0:
  minB = 0

if totalBoosts > 0:
  ax3.set_ylim([minB*0.8, maxB*1.2])


nameConverter = {
  'highest reliability': 'HCR',
  'lowest energy consumption': 'LEEC',
  'lowest retry count': 'LRC',
  'strongest signal': 'BASS'
}

newNames = []
for i, n in enumerate(sortedNames):
  newNames.append(nameConverter[n])

#colors = ['#336699', '#db7d2b', '#3b8e3d','#a01b1b']
#colors = ['#a6cee3','#1f78b4','#b2df8a','#33a02c']
colors = ['#2b83ba','#fdae61','#abdda4','#d7191c']

value_format_float = "{:.2f}"
for i, n in enumerate(sortedNames):
  bar = ax1.bar(i, sortedMeanRetries[i], bar_width, yerr=sortedStdRetries[i], capsize=10, label=n, color=colors[i])
  #label_bar(ax1, bar, value_format_float, is_inside=False, color="black")
  bar = ax2.bar(i, sortedMeanEnergy[i], bar_width, yerr=sortedStdEnergy[i], capsize=10, label=n, color=colors[i], ecolor='black')
  #label_bar(ax2, bar, value_format_float, is_inside=False, color="black")
  if totalBoosts > 0:
    bar = ax3.bar(i, sortedMeanBoosts[i], bar_width, yerr=sortedStdBoosts[i], capsize=10, label=n, color=colors[i], ecolor='black')

ax1.set_xticks(index)
ax1.set_xticklabels(([""]))
ax2.set_xticks(index)
ax2.set_xticklabels(([""]))

if totalBoosts > 0:
  ax3.set_xticks(index)
  ax3.set_xticklabels((newNames))
else:
  ax2.set_xticks(index)
  ax2.set_xticklabels((newNames))

if False:
  if totalBoosts > 0:
    for label in ax3.get_xmajorticklabels() + ax3.get_xmajorticklabels():
      label.set_rotation(20)
      label.set_horizontalalignment("right")
  else:
    for label in ax2.get_xmajorticklabels() + ax2.get_xmajorticklabels():
      label.set_rotation(20)
      label.set_horizontalalignment("right")

ax1.set_ylabel('Retries')
ax2.set_ylabel('Energy Cons.\n[' + energyUnit + ']')

if totalBoosts > 0:
  ax3.set_ylabel('Tx Boost')

plt.savefig(name + "_results.pdf", bbox_inches='tight')
